@@ -1,11 +1,43 @@
 """Paster Commands, for use with paster in your project
 
-The command(s) listed here are for use with Paste to enable easy creation of
-various core Pylons templates.
+.. highlight:: bash
 
-Currently available commands are::
+The following commands are made available via paster utilizing
+setuptools points discovery. These can be used from the command line
+when the directory is the Pylons project.
+
+Commands available:
+
+``controller``
+    Create a Controller and accompanying functional test
+``restcontroller``
+    Create a REST Controller and accompanying functional test
+``shell``
+    Open an interactive shell with the Pylons app loaded
+
+Example usage::
+    
+    ~/sample$ paster controller account
+    Creating /Users/ben/sample/sample/controllers/account.py
+    Creating /Users/ben/sample/sample/tests/functional/test_account.py
+    ~/sample$
+
+.. admonition:: How it Works
+
+    :command:`paster` is a command line script (from the PasteScript
+    package) that allows the creation of context sensitive commands.
+    :command:`paster` looks in the current directory for a 
+    ``.egg-info`` directory, then loads the ``paster_plugins.txt``
+    file.
+
+    Using setuptools entry points, :command:`paster` looks for
+    functions registered with setuptools as 
+    :func:`paste.paster_command`. These are defined in the entry_points
+    block in each packages :file:`setup.py` module.
+
+    This same system is used when running :command:`paster create` to
+    determine what templates are available when creating new projects.
 
-    controller, restcontroller, shell
 """
 import os
 import sys
@@ -16,15 +48,16 @@ import paste.deploy.config
 from paste.deploy import loadapp, appconfig
 from paste.script.command import Command, BadCommand
 from paste.script.filemaker import FileOp
-from paste.script.pluginlib import find_egg_info_dir
+from tempita import paste_script_template_renderer
 
+import pylons
 import pylons.util as util
 
 __all__ = ['ControllerCommand', 'RestControllerCommand', 'ShellCommand']
 
 def can_import(name):
-    """Attempt to __import__ the specified package/module, returning True when
-    succeeding, otherwise False"""
+    """Attempt to __import__ the specified package/module, returning
+    True when succeeding, otherwise False"""
     try:
         __import__(name)
         return True
@@ -32,9 +65,13 @@ def can_import(name):
         return False
 
 
-def is_minimal_template(package):
-    """Determine if the specified Pylons project (package) uses the Pylons
-    Minimal Tempalte"""
+def is_minimal_template(package, fail_fast=False):
+    """Determine if the specified Pylons project (package) uses the
+    Pylons Minimal Template.
+
+    fail_fast causes ImportErrors encountered during detection to be
+    raised.
+    """
     minimal_template = False
     try:
         # Check if PACKAGE.lib.base exists
@@ -44,10 +81,24 @@ def is_minimal_template(package):
             minimal_template = True
     except:
         # PACKAGE.lib.base exists but throws an error
-        pass
+        if fail_fast:
+            raise
     return minimal_template
 
 
+def defines_render(package):
+    """Determine if the specified Pylons project (package) defines a
+    render callable in their base module
+    """
+    base_module = (is_minimal_template(package) and package + '.controllers' or
+                   package + '.lib.base')
+    try:
+        base = __import__(base_module, globals(), locals(), ['__doc__'])
+    except:
+        return False
+    return callable(getattr(base, 'render', None))
+
+
 def validate_name(name):
     """Validate that the name for the controller isn't present on the
     path already"""
@@ -70,11 +121,21 @@ def validate_name(name):
     return True
 
 
+def check_controller_existence(base_package, directory, name): 
+    """Check if given controller already exists in project."""
+    filename = os.path.join(base_package, 'controllers', directory,
+                            name + '.py')
+    if os.path.exists(filename): 
+        raise BadCommand('Controller %s already exists.' %
+                         os.path.join(directory, name))
+
+
 class ControllerCommand(Command):
     """Create a Controller and accompanying functional test
 
     The Controller command will create the standard controller template
-    file and associated functional test to speed creation of controllers.
+    file and associated functional test to speed creation of
+    controllers.
 
     Example usage::
 
@@ -82,14 +143,15 @@ class ControllerCommand(Command):
         Creating yourproj/yourproj/controllers/comments.py
         Creating yourproj/yourproj/tests/functional/test_comments.py
 
-    If you'd like to have controllers underneath a directory, just include
-    the path as the controller name and the necessary directories will be
-    created for you::
+    If you'd like to have controllers underneath a directory, just
+    include the path as the controller name and the necessary
+    directories will be created for you::
 
         yourproj% paster controller admin/trackback
         Creating yourproj/controllers/admin
         Creating yourproj/yourproj/controllers/admin/trackback.py
         Creating yourproj/yourproj/tests/functional/test_admin_trackback.py
+
     """
     summary = __doc__.splitlines()[0]
     usage = '\n' + __doc__
@@ -109,8 +171,7 @@ class ControllerCommand(Command):
     def command(self):
         """Main command to create controller"""
         try:
-            file_op = FileOp(source_dir=os.path.join(
-                os.path.dirname(__file__), 'templates'))
+            file_op = FileOp(source_dir=('pylons', 'templates'))
             try:
                 name, directory = file_op.parse_path_name_args(self.args[0])
             except:
@@ -128,9 +189,13 @@ class ControllerCommand(Command):
 
             # Determine the module's import statement
             if is_minimal_template(base_package):
-                importstatement = "from %s.controllers import *" % base_package
+                importstatement = ('from %s.controllers import BaseController'
+                                   % base_package)
             else:
-                importstatement = "from %s.lib.base import *" % base_package
+                importstatement = ('from %s.lib.base import BaseController' %
+                                   base_package)
+            if defines_render(base_package):
+                importstatement += ', render'
 
             # Setup the controller
             fullname = os.path.join(directory, name)
@@ -139,17 +204,26 @@ class ControllerCommand(Command):
             if not fullname.startswith(os.sep):
                 fullname = os.sep + fullname
             testname = fullname.replace(os.sep, '_')[1:]
+            
+            module_dir = directory.replace('/', os.path.sep)
+            check_controller_existence(base_package, module_dir, name)
+            
             file_op.template_vars.update(
                 {'name': controller_name,
                  'fname': os.path.join(directory, name),
+                 'tmpl_name': name,
+                 'package':base_package,
                  'importstatement': importstatement})
             file_op.copy_file(template='controller.py_tmpl',
-                         dest=os.path.join('controllers', directory),
-                         filename=name)
+                              dest=os.path.join('controllers', directory),
+                              filename=name,
+                              template_renderer=paste_script_template_renderer)
             if not self.options.no_test:
-                file_op.copy_file(template='test_controller.py_tmpl',
-                             dest=os.path.join('tests', 'functional'),
-                             filename='test_'+testname)
+                file_op.copy_file(
+                    template='test_controller.py_tmpl',
+                    dest=os.path.join('tests', 'functional'),
+                    filename='test_' + testname,
+                    template_renderer=paste_script_template_renderer)
         except BadCommand, e:
             raise BadCommand('An error occurred. %s' % e)
         except:
@@ -160,15 +234,17 @@ class ControllerCommand(Command):
 class RestControllerCommand(Command):
     """Create a REST Controller and accompanying functional test
 
-    The RestController command will create a REST-based Controller file for use
-    with the map.resource REST-based dispatching. This template includes the
-    methods that map.resource dispatches to in addition to doc strings for
-    clarification on when the methods will be called.
+    The RestController command will create a REST-based Controller file
+    for use with the :meth:`~routes.mapper.Mapper.resource`
+    REST-based dispatching. This template includes the methods that
+    :meth:`~routes.mapper.Mapper.resource` dispatches to in
+    addition to doc strings for clarification on when the methods will
+    be called.
 
-    The first argument should be the singular form of the REST resource. The
-    second argument is the plural form of the word. If its a nested controller,
-    put the directory information in front as shown in the second example
-    below.
+    The first argument should be the singular form of the REST
+    resource. The second argument is the plural form of the word. If
+    its a nested controller, put the directory information in front as
+    shown in the second example below.
 
     Example usage::
 
@@ -176,14 +252,15 @@ class RestControllerCommand(Command):
         Creating yourproj/yourproj/controllers/comments.py
         Creating yourproj/yourproj/tests/functional/test_comments.py
 
-    If you'd like to have controllers underneath a directory, just include
-    the path as the controller name and the necessary directories will be
-    created for you::
+    If you'd like to have controllers underneath a directory, just
+    include the path as the controller name and the necessary
+    directories will be created for you::
 
         yourproj% paster restcontroller admin/tracback admin/trackbacks
         Creating yourproj/controllers/admin
         Creating yourproj/yourproj/controllers/admin/trackbacks.py
         Creating yourproj/yourproj/tests/functional/test_admin_trackbacks.py
+
     """
     summary = __doc__.splitlines()[0]
     usage = '\n' + __doc__
@@ -203,13 +280,13 @@ class RestControllerCommand(Command):
     def command(self):
         """Main command to create controller"""
         try:
-            file_op = FileOp(source_dir=os.path.join(
-                os.path.dirname(__file__), 'templates'))
+            file_op = FileOp(source_dir=('pylons', 'templates'))
             try:
                 singularname, singulardirectory = \
                     file_op.parse_path_name_args(self.args[0])
                 pluralname, pluraldirectory = \
                     file_op.parse_path_name_args(self.args[1])
+
             except:
                 raise BadCommand('No egg_info directory was found')
 
@@ -220,16 +297,24 @@ class RestControllerCommand(Command):
                     'Your controller name should not be the same as '
                     'the package name %r.'% base_package)
             # Validate the name
-            for name in [singularname, pluralname]:
+            for name in [pluralname]:
                 name = name.replace('-', '_')
                 validate_name(name)
 
             # Determine the module's import statement
             if is_minimal_template(base_package):
-                importstatement = "from %s.controllers import *" % base_package
+                importstatement = ('from %s.controllers import BaseController'
+                                   % base_package)
             else:
-                importstatement = "from %s.lib.base import *" % base_package
-
+                importstatement = ('from %s.lib.base import BaseController' %
+                                   base_package)
+            if defines_render(base_package):
+                importstatement += ', render'
+            
+            
+            module_dir = pluraldirectory.replace('/', os.path.sep)
+            check_controller_existence(base_package, module_dir, name)
+            
             # Setup the controller
             fullname = os.path.join(pluraldirectory, pluralname)
             controller_name = util.class_name_from_module_name(
@@ -239,15 +324,17 @@ class RestControllerCommand(Command):
             testname = fullname.replace(os.sep, '_')[1:]
 
             nameprefix = ''
+            path = ''
             if pluraldirectory:
                 nameprefix = pluraldirectory.replace(os.path.sep, '_') + '_'
-
+                path = pluraldirectory + '/'
+                
             controller_c = ''
             if nameprefix:
                 controller_c = ", controller='%s', \n\t" % \
                     '/'.join([pluraldirectory, pluralname])
-                controller_c += "path_prefix='/%s', name_prefix='%s_'" % \
-                    (pluraldirectory, pluraldirectory)
+                controller_c += "path_prefix='/%s', name_prefix='%s'" % \
+                    (pluraldirectory, nameprefix)
             command = "map.resource('%s', '%s'%s)\n" % \
                 (singularname, pluralname, controller_c)
 
@@ -257,6 +344,8 @@ class RestControllerCommand(Command):
                  'singularname': singularname,
                  'name': controller_name,
                  'nameprefix': nameprefix,
+                 'package':base_package,
+                 'path':path,
                  'resource_command': command.replace('\n\t', '\n%s#%s' % \
                                                          (' '*4, ' '*9)),
                  'fname': os.path.join(pluraldirectory, pluralname),
@@ -268,12 +357,15 @@ class RestControllerCommand(Command):
                                  "this:\n\n")
             resource_command += command
             file_op.copy_file(template='restcontroller.py_tmpl',
-                         dest=os.path.join('controllers', pluraldirectory),
-                         filename=pluralname)
+                              dest=os.path.join('controllers', pluraldirectory),
+                              filename=pluralname,
+                              template_renderer=paste_script_template_renderer)
             if not self.options.no_test:
-                file_op.copy_file(template='test_controller.py_tmpl',
-                             dest=os.path.join('tests', 'functional'),
-                             filename='test_'+testname)
+                file_op.copy_file(
+                    template='test_restcontroller.py_tmpl',
+                    dest=os.path.join('tests', 'functional'),
+                    filename='test_' + testname,
+                    template_renderer=paste_script_template_renderer)
             print resource_command
         except BadCommand, e:
             raise BadCommand('An error occurred. %s' % e)
@@ -294,6 +386,7 @@ class ShellCommand(Command):
     Example::
 
         $ paster shell my-development.ini
+
     """
     summary = __doc__.splitlines()[0]
     usage = '\n' + __doc__
@@ -312,7 +405,8 @@ class ShellCommand(Command):
                       action='count',
                       dest='quiet',
                       default=0,
-                      help="Do not load logging configuration from the config file")
+                      help=("Do not load logging configuration from the "
+                            "config file"))
 
     def command(self):
         """Main command to create a new shell"""
@@ -332,6 +426,10 @@ class ShellCommand(Command):
         here_dir = os.getcwd()
         locs = dict(__name__="pylons-admin")
 
+        if not self.options.quiet:
+            # Configure logging from the config file
+            self.logging_file_config(config_file)
+        
         # XXX: Note, initializing CONFIG here is Legacy support. pylons.config
         # will automatically be initialized and restored via the registry
         # restorer along with the other StackedObjectProxys
@@ -363,37 +461,39 @@ class ShellCommand(Command):
         # Restore the state of the Pylons special objects
         # (StackedObjectProxies)
         paste.registry.restorer.restoration_begin(request_id)
-
-        # Determine the package name from the .egg-info top_level.txt.
-        egg_info = find_egg_info_dir(here_dir)
-        f = open(os.path.join(egg_info, 'top_level.txt'))
-        packages = [l.strip() for l in f.readlines()
-                    if l.strip() and not l.strip().startswith('#')]
-        f.close()
+                
+        # Determine the package name from the pylons.config object
+        pkg_name = pylons.config['pylons.package']
 
         # Start the rest of our imports now that the app is loaded
-        found_base = False
-        for pkg_name in packages:
-            # Import all objects from the base module
+        if is_minimal_template(pkg_name, True):
+            model_module = None
+            helpers_module = pkg_name + '.helpers'
+            base_module = pkg_name + '.controllers'
+        else:
+            model_module = pkg_name + '.model'
+            helpers_module = pkg_name + '.lib.helpers'
             base_module = pkg_name + '.lib.base'
-            found_base = can_import(base_module)
-            if not found_base:
-                # Minimal template
-                base_module = pkg_name + '.controllers'
-                found_base = can_import(base_module)
 
-            if found_base:
-                break
+        if model_module and can_import(model_module):
+            locs['model'] = sys.modules[model_module]
 
-        if not found_base:
-            raise ImportError("Could not import base module. Are you sure "
-                              "this is a Pylons app?")
+        if can_import(helpers_module):
+            locs['h'] = sys.modules[helpers_module]
+
+        exec ('from pylons import app_globals, c, config, g, request, '
+              'response, session, tmpl_context, url') in locs
+        exec ('from pylons.controllers.util import abort, redirect') in locs
+        exec 'from pylons.i18n import _, ungettext, N_' in locs
+        exec 'from pylons.templating import render' in locs
+        
+        # Import all objects from the base module
+        __import__(base_module)
 
         base = sys.modules[base_module]
         base_public = [__name for __name in dir(base) if not \
                        __name.startswith('_') or __name == '_']
-        for name in base_public:
-            locs[name] = getattr(base, name)
+        locs.update((name, getattr(base, name)) for name in base_public)
         locs.update(dict(wsgiapp=wsgiapp, app=test_app))
 
         mapper = tresponse.config.get('routes.map')
@@ -409,10 +509,6 @@ class ShellCommand(Command):
         banner += "  %-10s -  %s\n" % ('app',
             'paste.fixture wrapped around wsgiapp')
 
-        if not self.options.quiet:
-            # Configure logging from the config file
-            self.logging_file_config(config_file)
-        
         try:
             if self.options.disable_ipython:
                 raise ImportError()
@@ -428,7 +524,9 @@ class ShellCommand(Command):
                 paste.registry.restorer.restoration_end()
         except ImportError:
             import code
-            newbanner = "Pylons Interactive Shell\nPython %s\n\n" % sys.version
+            py_prefix = sys.platform.startswith('java') and 'J' or 'P'
+            newbanner = "Pylons Interactive Shell\n%sython %s\n\n" % \
+                (py_prefix, sys.version)
             banner = newbanner + banner
             shell = code.InteractiveConsole(locals=locs)
             try:
