@@ -1,9 +1,17 @@
-"""Configuration setup for templating systems and Paste error
-middleware
+"""Configuration object and defaults setup
+
+The PylonsConfig object is initialized in pylons projects inside the
+:file:`config/environment.py` module. Importing the :data:`config`
+object from module causes the PylonsConfig object to be created, and
+setup in  app-safe manner so that multiple apps being setup avoid
+conflicts.
+
+After importing :data:`config`, the project should then call
+:meth:`~PylonsConfig.init_app` with the appropriate options to setup
+the configuration. In the config data passed with
+:meth:`~PylonsConfig.init_app`, various defaults are set use with Paste
+and Routes.
 
-This module supplies PylonsConfig which handles setting up defaults
-for templating systems, Paste errorware, and prefixing Routes if
-necessary.
 """
 import copy
 import logging
@@ -12,6 +20,7 @@ import warnings
 
 from paste.config import DispatchingConfig
 from paste.deploy.converters import asbool
+from webhelpers.mimehelper import MIMETypes
 
 import pylons.legacy
 import pylons.templating
@@ -28,30 +37,30 @@ response_defaults = dict(content_type='text/html',
 log = logging.getLogger(__name__)
 
 
-class PylonsConfig(DispatchingConfig):
+config = DispatchingConfig()
+
+class PylonsConfig(dict):
     """Pylons configuration object
 
-    The Pylons configuration object is a per-application instance object
-    that retains the information regarding the global and app conf's as
-    well as per-application instance specific data such as the mapper, the
-    paths for this instance, and the myghty configuration.
+    The Pylons configuration object is a per-application instance
+    object that retains the information regarding the global and app
+    conf's as well as per-application instance specific data such as
+    the mapper, and the paths for this instance.
 
-    The config object is available in your application as the Pylons global
-    ``pylons.config``. An example usage:
-
-    .. code-block :: Python
+    The config object is available in your application as the Pylons
+    global :data:`pylons.config`. For example::
 
         from pylons import config
 
         template_paths = config['pylons.paths']['templates']
 
-    There's several useful keys of the config object most people will be
-    interested in:
+    There's several useful keys of the config object most people will
+    be interested in:
 
     ``pylons.template_options``
-        Full dict of template options that any TG compatible plugin should
-        be able to parse. Comes with basic config needed for Myghty, Kid,
-        and Mako.
+        Full dict of template options that any TG compatible plugin
+        should be able to parse. Comes with basic config needed for
+        Genshi, Mako, Myghty, and Kid.
     ``pylons.paths``
         A dict of absolute paths that were defined in the applications
         ``config/environment.py`` module.
@@ -61,30 +70,36 @@ class PylonsConfig(DispatchingConfig):
         PylonsApp will use them from environ rather than using default
         middleware from Beaker. Valid keys are: ``session, cache``
     ``pylons.template_engines``
-        List of template engines to configure. The first one in the list will
-        be configured as the default template engine. Each item in the list is
-        a dict indicating how to configure the template engine with keys:
-        ``engine``, ``template_root``, ``template_options``, and ``alias``
+        List of template engines to configure. The first one in the
+        list will be configured as the default template engine. Each
+        item in the list is a dict indicating how to configure the
+        template engine with keys:
+        
+        ``engine``, ``template_root``, ``template_options``, and 
+        ``alias``
     ``pylons.default_charset``
         Deprecated: Use the response_settings dict instead.
         Default character encoding specified to the browser via the
         'charset' parameter of the HTTP response's Content-Type header.
     ``pylons.strict_c``
-        Whether or not the ``c`` object should throw an attribute error when
-        access is attempted to an attribute that doesn't exist.
+        Whether or not the ``c`` object should throw an attribute error
+        when access is attempted to an attribute that doesn't exist.
     ``pylons.request_options``
-        A dict of Content-Type related default settings for new instances of
-        ``paste.wsgiwrappers.WSGIRequest``. May contain the values ``charset``
-        and ``errors`` and ``decode_param_names``. Overrides the Pylons default
-        values specified by the ``request_defaults`` dict.
+        A dict of Content-Type related default settings for new
+        instances of :class:`~pylons.controllers.util.Request`. May
+        contain the values ``charset`` and ``errors`` and 
+        ``decode_param_names``. Overrides the Pylons default values
+        specified by the ``request_defaults`` dict.
     ``pylons.response_options``
-        A dict of Content-Type related default settings for new instances of
-        ``pylons.Response``. May contain the values ``content_type``,
-        ``charset`` and ``errors``. Overrides the Pylons default values
-        specified by the ``response_defaults`` dict.
+        A dict of Content-Type related default settings for new 
+        instances of :class:`~pylons.controllers.util.Response`. May
+        contain the values ``content_type``, ``charset`` and
+        ``errors``. Overrides the Pylons default values specified by
+        the ``response_defaults`` dict.
     ``routes.map``
-        Mapper object used for Routing. Yes, it is possible to add routes
-        after your application has started running.
+        Mapper object used for Routing. Yes, it is possible to add
+        routes after your application has started running.
+    
     """
     defaults = {
         'debug': False,
@@ -94,15 +109,36 @@ class PylonsConfig(DispatchingConfig):
                          'templates': [],
                          'static_files': None},
         'pylons.db_engines': {},
-        'pylons.environ_config': {},
+        'pylons.environ_config': dict(session='beaker.session', 
+                                      cache='beaker.cache'),
         'pylons.g': None,
         'pylons.h': None,
         'pylons.request_options': request_defaults.copy(),
         'pylons.response_options': response_defaults.copy(),
-        'pylons.strict_c': False,
+        'pylons.strict_tmpl_context': False,
+        'pylons.tmpl_context_attach_args': True,
         'buffet.template_engines': [],
         'buffet.template_options': {},
     }
+    
+    def __setitem__(self, name, value):
+        if name in ['pylons.strict_c', 'pylons.c_attach_args']:
+            new_name = name.replace('strict_c', 'strict_tmpl_context')
+            new_name = name.replace('c_attach', 'tmpl_context_attach')
+            warnings.warn(pylons.legacy.c_attrib_moved % \
+                          (name, new_name),
+                          DeprecationWarning, 3)
+            name = new_name
+        dict.__setitem__(self, name, value)
+
+    def __getitem__(self, name):
+        if name in ['pylons.strict_c', 'pylons.c_attach_args']:
+            new_name = name.replace('c', 'tmpl_context')
+            warnings.warn(pylons.legacy.c_attrib_moved % \
+                          (name, new_name),
+                          DeprecationWarning, 3)
+            name = new_name
+        return dict.__getitem__(self, name)
 
     def __getattr__(self, name):
         # Backwards compatibility
@@ -116,21 +152,27 @@ class PylonsConfig(DispatchingConfig):
                     setattr(self, name, value)
             return FakeConfig
         else:
-            conf_dict = self.current_conf()
+            conf_dict = self
 
-            # Backwards compat for when the option is now in the dict, and
-            # access was attempted via attribute
+            # Backwards compat for when the option is now in the dict, 
+            # and access was attempted via attribute
             for prefix in ('', 'pylons.', 'buffet.', 'routes.'):
                 full_name = prefix + name
                 if full_name in conf_dict:
                     warnings.warn(pylons.legacy.config_attr_moved % \
-                                      (name, full_name), DeprecationWarning, 3)
+                                   (name, full_name), DeprecationWarning, 3)
                     return conf_dict[full_name]
+            if name in ['pylons.strict_c', 'pylons.c_attach_args']:
+                new_name = name.replace('c', 'tmpl_context')
+                warnings.warn(pylons.legacy.c_attrib_moved % \
+                              (name, new_name),
+                              DeprecationWarning, 3)
+                name = new_name
             if name == 'request_defaults':
                 return request_defaults
             elif name == 'response_defaults':
                 return response_defaults
-            return getattr(conf_dict, name)
+            return conf_dict[name]
 
     def load_environment(self, tmpl_options=None, map=None, paths=None,
                          environ_config=None, default_charset=None,
@@ -153,9 +195,11 @@ class PylonsConfig(DispatchingConfig):
         if response_settings:
             conf['pylons.response_options'].update(response_settings)
 
+        if environ_config is not None:
+            conf['pylons.environ_config'] = environ_config
+
         conf['routes.map'] = map
         conf['pylons.paths'] = paths or {}
-        conf['pylons.environ_config'] = environ_config or {}
         conf['pylons.strict_c'] = strict_c
 
         if default_charset:
@@ -166,8 +210,8 @@ class PylonsConfig(DispatchingConfig):
         self['environment_load'] = conf
 
     def add_template_engine(self, engine, root, options=None, alias=None):
-        """Add additional template engines for configuration on Pylons WSGI
-        init.
+        """Add additional template engines for configuration on Pylons
+        WSGI init.
 
         ``engine``
             The name of the template engine
@@ -176,29 +220,32 @@ class PylonsConfig(DispatchingConfig):
             Template root for the engine
 
         ``options``
-            Dict of additional options used during engine initialization, if
-            not provided, default to using the template_options dict.
+            Dict of additional options used during engine
+            initialization, if not provided, default to using the
+            template_options dict.
 
         ``alias``
-            Name engine should respond to when actually used. This allows for
-            multiple configurations of the same engine and lets you alias the
-            additional ones to other names.
+            Name engine should respond to when actually used. This
+            allows for multiple configurations of the same engine and
+            lets you alias the additional ones to other names.
 
         Example of Kid addition:
 
-        .. code-block:: Python
+        .. code-block:: python
 
             # In yourproj/middleware.py
             # ...
             config.init_app(global_conf, app_conf, package='yourproj')
 
             # Load additional template engines
-            kidopts = {'kid.assume_encoding':'utf-8', 'kid.encoding':'utf-8'}
-            config.add_template_engine('kid', 'yourproj.kidtemplates', kidopts)
+            kidopts = {'kid.assume_encoding':'utf-8', 
+                       'kid.encoding':'utf-8'}
+            config.add_template_engine('kid', 'yourproj.kidtemplates', 
+                                       kidopts)
 
         Example of changing the default template engine:
 
-        .. code-block:: Python
+        .. code-block:: python
 
             # In yourproj/middleware.py
             # ...
@@ -208,12 +255,17 @@ class PylonsConfig(DispatchingConfig):
             old_default = config.template_engines.pop()
 
             # Load additional template engines
-            kidopts = {'kid.assume_encoding':'utf-8', 'kid.encoding':'utf-8'}
-            config.add_template_engine('kid', 'yourproj.kidtemplates', kidopts)
+            kidopts = {'kid.assume_encoding':'utf-8', 
+                       'kid.encoding':'utf-8'}
+            config.add_template_engine('kid', 'yourproj.kidtemplates', 
+                                       kidopts)
 
             # Add old default as additional engine
             config.template_engines.append(old_default)
+        
         """
+        warnings.warn("add_template_engine is deprecated, use custom render"
+                      " functions instead", DeprecationWarning, 3)
         if not options:
             options = self['buffet.template_options']
         config = dict(engine=engine, template_root=root,
@@ -227,33 +279,39 @@ class PylonsConfig(DispatchingConfig):
         """Initialize configuration for the application
         
         .. note
-            This *must* be called at least once, as soon as possible tosetup 
-            all the configuration options.
+            This *must* be called at least once, as soon as possible 
+            to setup all the configuration options.
         
-        ``global_config``
+        ``global_conf``
             Several options are expected to be set for a Pylons web
-            application. They will be loaded from the global_config which has
-            the main Paste options. If ``debug`` is not enabled as a global
-            config option, the following option *must* be set:
+            application. They will be loaded from the global_config 
+            which has the main Paste options. If ``debug`` is not 
+            enabled as a global config option, the following option
+            *must* be set:
 
             * error_to - The email address to send the debug error to
 
             The optional config options in this case are:
 
-            * smtp_server - The SMTP server to use, defaults to 'localhost'
+            * smtp_server - The SMTP server to use, defaults to 
+              'localhost'
             * error_log - A logfile to write the error to
-            * error_subject_prefix - The prefix of the error email subject
+            * error_subject_prefix - The prefix of the error email
+              subject
             * from_address - Whom the error email should be from
         ``app_conf``
             Defaults supplied via the [app:main] section from the Paste
-            config file. ``load_config`` only cares about whether a 'prefix'
-            option is set, if so it will update Routes to ensure URL's take
-            that into account.
+            config file. ``load_config`` only cares about whether a 
+            'prefix' option is set, if so it will update Routes to
+            ensure URL's take that into account.
         ``package``
-            The name of the application package, to be stored in the app_conf.
-        ``template_engine``
-            Declare the default template engine to setup. Choices are kid,
-            genshi, mako (the default), and pylonsmyghty.
+            The name of the application package, to be stored in the 
+            app_conf.
+        
+        .. versionchanged:: 0.9.7
+            ``template_engine`` is no longer required, and can be set
+            to :data:`None` to avoid loading the default one.
+                
         """
         log.debug("Initializing configuration, package: '%s'", package)
         conf = global_conf.copy()
@@ -276,12 +334,14 @@ class PylonsConfig(DispatchingConfig):
             warnings.warn(pylons.legacy.root_path, DeprecationWarning, 2)
             paths['root'] = paths['root_path']
         
-        log.debug("Pushing process configuration")
-        self.push_process_config(conf)
-        self.set_defaults(template_engine)
+        self.update(conf)
+        self.set_defaults()
     
-    def set_defaults(self, template_engine):
-        conf = self.current_conf()
+    def set_defaults(self, template_engine=None):
+        conf = self
+        
+        # Load the MIMETypes with its default types
+        MIMETypes.init()
         
         # Ensure all the keys from defaults are present, load them if not
         for key, val in copy.deepcopy(PylonsConfig.defaults).iteritems():
@@ -398,7 +458,8 @@ class PylonsConfig(DispatchingConfig):
             self.add_template_engine(template_engine, '%s.templates' % 
                                      conf['pylons.package'])
         
-        log.debug("Loaded %s template engine as the default template renderer", template_engine)
+        log.debug("Loaded %s template engine as the default template "
+                  "renderer", template_engine)
         
         conf['pylons.cache_dir'] = conf.pop('cache_dir', 
                                             conf['app_conf'].get('cache_dir'))
@@ -406,11 +467,11 @@ class PylonsConfig(DispatchingConfig):
         conf['pylons.errorware'] = errorware
 
 
-config = PylonsConfig()
+pylons_config = PylonsConfig()
 
 
 # Push an empty config so all accesses to config at import time have something
 # to look at and modify. This config will be merged with the app's when it's
 # built in the paste.app_factory entry point.
-initial_config = copy.deepcopy(PylonsConfig.defaults)
-config.push_process_config(initial_config)
+pylons_config.update(copy.deepcopy(PylonsConfig.defaults))
+config.push_process_config(pylons_config)
