@@ -84,10 +84,11 @@ class WSGIController(object):
                 
         log_debug = self._pylons_log_debug
         c = self._py_object.c
+        environ = self._py_object.request.environ
         args = None
         
         if argspec[2]:
-            if self._py_object.config['pylons.c_attach_args']:
+            if self._py_object.config['pylons.tmpl_context_attach_args']:
                 for k, val in kargs.iteritems():
                     setattr(c, k, val)
             args = kargs
@@ -97,7 +98,7 @@ class WSGIController(object):
                                   and 1 or 0:]
             for name in argnames:
                 if name in kargs:
-                    if self._py_object.config['pylons.c_attach_args']:
+                    if self._py_object.config['pylons.tmpl_context_attach_args']:
                         setattr(c, name, kargs[name])
                     args[name] = kargs[name]
         if log_debug:
@@ -112,6 +113,9 @@ class WSGIController(object):
                           httpe.wsgi_response.code, exc_info=True)
             result = httpe
             
+            # Store the exception in the environ
+            environ['pylons.controller.exception'] = httpe
+            
             # 304 Not Modified's shouldn't have a content-type set
             if result.wsgi_response.status_int == 304:
                 result.wsgi_response.headers.pop('Content-Type', None)
@@ -124,7 +128,7 @@ class WSGIController(object):
             warnings.warn("Raising a paste.httpexceptions.HTTPException is "
                           "deprecated, use webob.exc.HTTPException instead",
                           DeprecationWarning, 2)
-            result = httpe.response(pylons.request.environ)
+            result = httpe.response(environ)
             result.headers.pop('Content-Type')
             result._exception = True
 
@@ -207,9 +211,9 @@ class WSGIController(object):
             if log_debug:
                 log.debug("Merging pylons.response headers into "
                           "start_response call, status: %s", status)
-            for header in response.headerlist:
-                if header[0] == 'Set-Cookie' or header[0].startswith('X-'):
-                    headers.append(header)
+            headers.extend(header for header in response.headerlist
+                           if header[0] == 'Set-Cookie' or
+                           header[0].startswith('X-'))
             return start_response(status, headers, exc_info)
         self.start_response = repl_start_response
         
