@@ -4,11 +4,10 @@ import types
 import warnings
 
 from paste.registry import StackedObjectProxy
-from paste.wsgiwrappers import WSGIResponse
 
 import pylons
 import pylons.decorators
-from pylons.controllers import Controller as OrigController
+from pylons.controllers.util import Response as PylonsResponse
 from pylons.util import deprecated, func_move
 
 __all__ = ['load_h']
@@ -17,6 +16,10 @@ config_attr_moved = (
     "The attribute 'config.%s' has moved to the pylons.config dictionary: "
     "Please access it via pylons.config['%s']")
 
+c_attrib_moved = (
+"Setting or accessing %s is deprecated. Set / Access it as %s instead."
+)
+
 config_load_environment = (
 "The pylons.config.Config object is deprecated. Please load the environment "
 "configuration via the pylons.config object in config/environment.py instead, "
@@ -52,44 +55,11 @@ Add the following lines to the end of the load_environment function:
     config['pylons.response_options']['charset'] = '%(charset)s'
 """)
 
-g_confargs = (
-"Handling conf arguments in your app_globals __init__ function is no longer "
-"required. Please update your config/app_globals.py with:"
-"""
-    from pylons import config
-
-    class Globals(object):
-        def __init__(self):
-            pass
-
-And use the config object in your Globals instance.
-""")
-
-helpers_and_g_warning = (
-"Pylons 0.9.6 and above projects must explicitly initialize the helpers and g "
-"objects in their config/environment.py. Please no longer specify the helpers "
-"and g keyword arguments to PylonsApp in your config/middleware.py: instead, "
-"update your config/environment.py with:"
-"""
-
-    from pylons import config
-
-    import %(package)s.lib.app_globals as app_globals
-    import %(package)s.lib.helpers
-
-And add the following lines to the load_environment function:
-
-    # Initialize config with the basic options
-    config.init_app(global_conf, app_conf, package='%(package)s',
-                    template_engine='%(template_engine)s', paths=paths)
-
-    config['pylons.g'] = app_globals.Globals()
-    config['pylons.h'] = %(package)s.lib.helpers
-    config['routes.map'] = make_map()
-
-See the default config/environment.py created via the "paster create -t pylons"
-command for a full example.
-""")
+error_template_warning = (
+"""The 'error_template' errorware argument for customizing EvalException is \
+deprecated, please remove it. To customize EvalException's HTML, setup your \
+own EvalException and ErrorMiddlewares instead of using ErrorHandler."""
+)
 
 log_warning = (
 'The log function is deprecated. Use the logging module instead')
@@ -130,17 +100,32 @@ uses h):
     from pylons import h
 """)
 
-redirect_response_warning = (
-"redirect_to's _response argument will be deprecated (trigger a "
-"DeprecationWarning) in Pylons 0.9.7, and support for it will be removed in a "
-"future version of Pylons. Please use pylons.response prior to redirecting "
-"instead")
+pylons_c_warning = (
+"pylons.c is deprecated. Your project should import pylons.tmpl_context as c"
+"""instead:
+
+Before:
+    from pylons import c
+
+After:
+    from pylons import tmpl_context as c
+"""
+)
+
+pylons_g_warning = (
+"pylons.g is deprecated. Your project should import pylons.app_globals "
+"If desired, it can be imported as 'g', but its recommended to refer to "
+"it as app_globals for clarity"
+)
+
+pylons_buffet_warning = (
+"pylons.buffet is deprecated and will be removed in Pylons 1.0 entirely. "
+"The new render functions should be used instead."
+)
 
 render_response_warning = (
-"render_response will be deprecated (trigger a DeprecationWarning) in "
-"Pylons 0.9.7, and support for it will be removed in a future version of "
-"Pylons. Please return the response content directly (via the render "
-"function) instead")
+"render_response is deprecated, please return the response content directly "
+"(via the render function) instead")
 
 root_path = (
 "paths['root_path'] has been moved to paths['root'], please update your "
@@ -171,29 +156,25 @@ jsonify = deprecated(pylons.decorators.jsonify,
                      func_move('pylons.jsonify',
                                moved_to='pylons.decorators.jsonify'))
 
-controller_warning = ('pylons.Controller has been moved to '
-                      'pylons.controllers.Controller.')
-class Controller(OrigController):
-    def __init__(self, *args, **kwargs):
-        warnings.warn(controller_warning, DeprecationWarning, 2)
-        OrigController.__init__(self, *args, **kwargs)
-
-    def __call__(self, *args, **kwargs):
-        warnings.warn(controller_warning, DeprecationWarning, 2)
-        return OrigController.__call__(self, *args, **kwargs)
-
 class DeprecatedStackedObjectProxy(StackedObjectProxy):
-    def _current_obj(*args, **kwargs):
-        warnings.warn(pylons_h_warning, DeprecationWarning, 3)
-        return StackedObjectProxy._current_obj(*args, **kwargs)
-h = DeprecatedStackedObjectProxy(name="h")
+    def __init__(self, *args, **kwargs):
+        self.__dict__['__warning'] = kwargs.pop('warning')
+        StackedObjectProxy.__init__(self, *args, **kwargs)
+
+    def _current_obj(self, *args, **kwargs):
+        warnings.warn(self.__dict__['__warning'], DeprecationWarning, 3)
+        return StackedObjectProxy._current_obj(self, *args, **kwargs)
+
+h = DeprecatedStackedObjectProxy(name="h", warning=pylons_h_warning)
+c = DeprecatedStackedObjectProxy(name="tmpl_context", warning=pylons_c_warning)
+g = DeprecatedStackedObjectProxy(name="app_globals", warning=pylons_g_warning)
+buffet = DeprecatedStackedObjectProxy(name="buffet", warning=pylons_buffet_warning)
 
 response_warning = (
-"Returning a Response object from a controller will be deprecated (trigger a "
-"full DeprecationWarning) in Pylons 0.9.7, and support for it will be removed "
-"in a future version of Pylons. Please return the response content directly "
-"and or use pylons.response instead")
-class Response(WSGIResponse):
+"Returning a Response object from a controller is deprecated, and support for "
+"it will be removed in a future version of Pylons. Please return the response "
+"content directly and or use pylons.response instead")
+class Response(PylonsResponse):
     def __init__(self, *args, **kwargs):
-        warnings.warn(response_warning, PendingDeprecationWarning, 2)
-        WSGIResponse.__init__(self, *args, **kwargs)
+        warnings.warn(response_warning, DeprecationWarning, 2)
+        PylonsResponse.__init__(self, *args, **kwargs)
