@@ -14,12 +14,11 @@ from routes import request_config
 from webob.exc import HTTPFound, HTTPNotFound
 
 import pylons
-import pylons.legacy
 import pylons.templating
 from pylons.controllers.util import Request, Response
 from pylons.i18n.translation import _get_translator
-from pylons.util import AttribSafeContextObj, ContextObj, PylonsContext, \
-    class_name_from_module_name
+from pylons.util import (AttribSafeContextObj, ContextObj, PylonsContext,
+                         class_name_from_module_name)
 
 __all__ = ['PylonsApp']
 
@@ -44,7 +43,7 @@ class PylonsApp(object):
     approach.
     
     """
-    def __init__(self, **kwargs):
+    def __init__(self, config=None, **kwargs):
         """Initialize a base Pylons WSGI application
         
         The base Pylons WSGI application requires several keywords, the
@@ -52,7 +51,7 @@ class PylonsApp(object):
         provided then h will be None.
         
         """
-        self.config = config = pylons.config._current_obj()
+        self.config = config = config or pylons.config._current_obj()
         package_name = config['pylons.package']
         self.helpers = config['pylons.h']
         self.globals = config.get('pylons.app_globals') or config['pylons.g']
@@ -69,7 +68,7 @@ class PylonsApp(object):
             log.debug("Raising redirect to %s", url)
             raise HTTPFound(location=url)
         self.redirect_to = redirect_to
-        
+
         # Initialize Buffet and all our template engines, default engine is the
         # first in the template_engines list
         if config.get('buffet.template_engines'):
@@ -86,7 +85,7 @@ class PylonsApp(object):
                                     alias=e['alias'], **e['template_options'])
         else:
             self.buffet = None
-        
+                        
         # Cache some options for use during requests
         self._session_key = self.environ_config.get('session', 'beaker.session')
         self._cache_key = self.environ_config.get('cache', 'beaker.cache')
@@ -163,18 +162,22 @@ class PylonsApp(object):
         registry.register(pylons.request, pylons_obj.request)
         
         registry.register(pylons.app_globals, self.globals)
+        registry.register(pylons.g, self.globals)
         registry.register(pylons.config, self.config)
         registry.register(pylons.h, self.helpers or \
                           pylons.legacy.load_h(self.package_name))
-        registry.register(pylons.c, pylons_obj.c)
+        registry.register(pylons.tmpl_context, pylons_obj.tmpl_context)
+        registry.register(pylons.c, pylons_obj.tmpl_context)
         registry.register(pylons.translator, pylons_obj.translator)
-        
+
         if self.buffet:
-            registry.register(pylons.buffet, self.buffet)
+            registry.register(pylons.buffet, self.buffet)        
         if 'session' in pylons_obj.__dict__:
             registry.register(pylons.session, pylons_obj.session)
         if 'cache' in pylons_obj.__dict__:
             registry.register(pylons.cache, pylons_obj.cache)
+        elif 'cache' in pylons_obj.app_globals.__dict__:
+            registry.register(pylons.cache, pylons_obj.app_globals.cache)
         
         if 'routes.url' in environ:
             registry.register(pylons.url, environ['routes.url'])
@@ -208,9 +211,9 @@ class PylonsApp(object):
         pylons_obj.config = self.config
         pylons_obj.request = req
         pylons_obj.response = response
-        pylons_obj.g = pylons_obj.app_globals = self.globals
+        pylons_obj.app_globals = self.globals
         pylons_obj.h = self.helpers
-        
+
         if self.buffet:
             pylons_obj.buffet = self.buffet
         
@@ -222,11 +225,11 @@ class PylonsApp(object):
         lang = self.config['lang']
         pylons_obj.translator = _get_translator(lang, pylons_config=self.config)
         
-        if self.config['pylons.strict_c']:
-            c = ContextObj()
+        if self.config['pylons.strict_tmpl_context']:
+            tmpl_context = ContextObj()
         else:
-            c = AttribSafeContextObj()
-        pylons_obj.c = c
+            tmpl_context = AttribSafeContextObj()
+        pylons_obj.tmpl_context = pylons_obj.c = tmpl_context
         
         econf = self.config['pylons.environ_config']
         if self._session_key in environ:
@@ -331,9 +334,9 @@ class PylonsApp(object):
         pylons_obj = environ['pylons.pylons']
         testenv['req'] = pylons_obj.request
         testenv['response'] = pylons_obj.response
-        testenv['tmpl_context'] = testenv['c'] = pylons_obj.c
+        testenv['tmpl_context'] = pylons_obj.tmpl_context
         testenv['app_globals'] = testenv['g'] = pylons_obj.app_globals
-        testenv['h'] = self.config['pylons.h'] or pylons_obj.h
+        testenv['h'] = self.config['pylons.h']
         testenv['config'] = self.config
         if hasattr(pylons_obj, 'session'):
             testenv['session'] = pylons_obj.session
