@@ -7,7 +7,7 @@ import pylons
 from pylons.controllers import WSGIController
 from pylons.controllers.util import redirect_to
 
-from __init__ import TestWSGIController, SetupCacheGlobal, ControllerWrap
+from __init__ import TestWSGIController, SetupCacheGlobal, ControllerWrap, TestMiddleware
 
 class BasicWSGIController(WSGIController):
     def __before__(self):
@@ -35,6 +35,10 @@ class BasicWSGIController(WSGIController):
         exc = status_map[301]
         raise exc('/elsewhere').exception
     
+    def use_customnotfound(self):
+        exc = status_map[404]
+        raise exc('Custom not found').exception
+    
     def header_check(self):
         pylons.response.headers['Content-Type'] = 'text/plain'
         return "Hello all!"
@@ -79,6 +83,7 @@ class TestBasicWSGI(TestWSGIController):
         self.baseenviron = {}
         app = ControllerWrap(BasicWSGIController)
         app = self.sap = SetupCacheGlobal(app, self.baseenviron)
+        app = TestMiddleware(app)
         app = RegistryManager(app)
         self.app = TestApp(app)
         
@@ -100,6 +105,15 @@ class TestBasicWSGI(TestWSGIController):
         resp = self.app.get('/', status=404)
         assert resp.status == 404
     
+    def test_404exception(self):
+        self.environ['paste.config']['global_conf']['debug'] = False
+        self.environ['pylons.routes_dict']['action'] = 'use_customnotfound'
+        resp = self.app.get('/', status=404)
+        assert 'pylons.controller.exception' in resp.environ
+        exc = resp.environ['pylons.controller.exception']
+        assert exc.detail == 'Custom not found'
+        assert resp.status == 404
+    
     def test_private_func(self):
         self.baseenviron['pylons.routes_dict']['action'] = '_private'
         resp = self.app.get('/', status=404)
