@@ -8,7 +8,10 @@ from pylons.controllers import WSGIController
 from pylons.decorators.secure import authenticate_form, csrf_detected_message
 from pylons.testutil import ControllerWrap, SetupCacheGlobal
 from routes import request_config
-from webhelpers.rails import secure_form_tag
+try:
+    import webhelpers.html.secure_form as secure_form
+except ImportError:
+    import webhelpers.pylonslib.secure_form as secure_form
 
 from __init__ import data_dir, TestWSGIController
 
@@ -22,12 +25,12 @@ except:
 class ProtectedController(WSGIController):
     def form(self):
         request_config().environ = request.environ
-        return secure_form_tag.authentication_token()
+        return secure_form.authentication_token()
 
+    @authenticate_form
     def protected(self):
         request_config().environ = request.environ
         return 'Authenticated'
-    protected = authenticate_form(protected)
 
 class TestAuthenticateFormDecorator(TestWSGIController):
     def setUp(self):
@@ -52,14 +55,14 @@ class TestAuthenticateFormDecorator(TestWSGIController):
 
         self.environ['pylons.routes_dict']['action'] = 'protected'
         response = self.app.post('/protected',
-                                 params={secure_form_tag.token_key: token},
+                                 params={secure_form.token_key: token},
                                  extra_environ=self.environ,
                                  expect_errors=True)
         assert 'Authenticated' in response
 
         self.environ['pylons.routes_dict']['action'] = 'protected'
         response = self.app.put('/protected',
-                                params={secure_form_tag.token_key: token},
+                                params={secure_form.token_key: token},
                                 extra_environ=self.environ,
                                 expect_errors=True)
         assert 'Authenticated' in response
