@@ -3,25 +3,34 @@ from paste.registry import RegistryManager
 
 from routes.middleware import RoutesMiddleware
 
+from pylons import url
 from pylons.decorators.secure import https
-
 from pylons.controllers import WSGIController
 from pylons.testutil import ControllerWrap, SetupCacheGlobal
 
 from __init__ import TestWSGIController
 
 class HttpsController(WSGIController):
+
+    @https('/pylons')
     def index(self):
         return 'index page'
-    index = https('/pylons')(index)
 
+    @https(controller='auth', action='login')
     def login(self):
         return 'login page'
-    login = https(controller='auth', action='login')(login)
 
+    @https(lambda: url(controller='auth', action='login'))
+    def login2(self):
+        return 'login2 page'
+
+    @https(lambda: url.current())
+    def secure(self):
+        return 'secure page'
+
+    @https()
     def get(self):
         return 'get page'
-    get = https()(get)
 
 class TestHttpsDecorator(TestWSGIController):
     def setUp(self):
@@ -52,7 +61,6 @@ class TestHttpsDecorator(TestWSGIController):
 
     def test_https_disallows_post(self):
         self.environ['pylons.routes_dict']['action'] = 'index'
-
         response = self.app.post('/index', status=405)
 
     def test_https_url_for_kwargs(self):
@@ -67,6 +75,30 @@ class TestHttpsDecorator(TestWSGIController):
         assert 'location' not in response.header_dict
         assert 'login page' in response
 
+    def test_https_callable(self):
+        self.environ['pylons.routes_dict']['action'] = 'login2'
+
+        response = self.app.get('/login2', status=302)
+        assert response.header_dict.get('location') == \
+            'https://localhost/auth/login'
+
+        self.environ['wsgi.url_scheme'] = 'https'
+        response = self.app.get('/login2', status=200)
+        assert 'location' not in response.header_dict
+        assert 'login2 page' in response
+
+    def test_https_callable_current(self):
+        self.environ['pylons.routes_dict']['action'] = 'secure'
+
+        response = self.app.get('/secure', status=302)
+        assert response.header_dict.get('location') == \
+            'https://localhost/secure'
+
+        self.environ['wsgi.url_scheme'] = 'https'
+        response = self.app.get('/secure', status=200)
+        assert 'location' not in response.header_dict
+        assert 'secure page' in response
+
     def test_https_redirect_to_self(self):
         self.environ['pylons.routes_dict']['action'] = 'get'
 
