@@ -11,7 +11,7 @@ from __init__ import TestWSGIController, SetupCacheGlobal, ControllerWrap
 
 class BaseXMLRPCController(XMLRPCController):
     foo = 'bar'
-
+    
     def userstatus(self):
         return 'basic string'
     userstatus.signature = [ ['string'] ]
@@ -20,7 +20,7 @@ class BaseXMLRPCController(XMLRPCController):
         "This method has a docstring"
         return dict(mess='a little somethin', a=1, b=[1,2,3], c=('all','the'))
     docs.signature = [ ['struct'] ]
-
+    
     def uni(self):
         "This method has a docstring"
         return dict(mess=u'A unicode string, oh boy')
@@ -46,10 +46,10 @@ class BaseXMLRPCController(XMLRPCController):
         has multiple lines
         in it"""
         return "hi all"
-
+    
     def _private(self):
         return 'private method'
-        
+    
 class TestXMLRPCController(TestWSGIController):
     def __init__(self, *args, **kargs):
         TestWSGIController.__init__(self, *args, **kargs)
@@ -59,7 +59,7 @@ class TestXMLRPCController(TestWSGIController):
         app = self.sap = SetupCacheGlobal(app, self.baseenviron)
         app = RegistryManager(app)
         self.app = TestApp(app)
-
+    
     def test_index(self):
         response = self.xmlreq('userstatus')
         assert response == 'basic string'
@@ -67,46 +67,45 @@ class TestXMLRPCController(TestWSGIController):
     def test_structure(self):
         response = self.xmlreq('docs')
         assert dict(mess='a little somethin', a=1, b=[1,2,3], c=['all','the']) == response
-        
+    
     def test_methodhelp(self):
         response = self.xmlreq('system.methodHelp', ('docs',))
         assert "This method has a docstring" in response
-
+    
     def test_methodhelp_with_structured_methodname(self):
         response = self.xmlreq('system.methodHelp', ('structured.methodname',))
         assert "This method has a docstring" in response
-
+    
     def test_methodsignature(self):
         response = self.xmlreq('system.methodSignature', ('docs',))
         assert [['struct']] == response
-
+    
     def test_methodsignature_with_structured_methodname(self):
         response = self.xmlreq('system.methodSignature', ('structured.methodname',))
         assert [['string', 'string']] == response
-
+    
     def test_listmethods(self):
         response = self.xmlreq('system.listMethods')
         assert response == ['docs', 'intargcheck', 'longdoc', 'nosig', 'structured.methodname', 'system.listMethods', 'system.methodHelp', 'system.methodSignature', 'uni', 'userstatus']    
-
+    
     def test_unicode(self):
         response = self.xmlreq('uni')
         assert 'A unicode string' in response['mess']
-
+    
     def test_unicode_method(self):
         data = xmlrpclib.dumps((), methodname=u'ОбсуждениеКомпаний')
-        self.response = response = self.app.post('/', params=data,
-                                                 extra_environ=dict(CONTENT_TYPE='text/xml'))
-        
+        self.response = response = self.app.post('/', params=data, extra_environ=dict(CONTENT_TYPE='text/xml'))
+    
     def test_badargs(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, 'system.methodHelp')
-
+    
     def test_badarity(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, 'system.methodHelp')
-
+    
     # Unsure whether this is actually picked up by xmlrpclib, but what the hey
     def test_bad_paramval(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, 'intargcheck', (12.5,))
-
+    
     def test_missingmethod(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, 'doesntexist')
     
@@ -133,12 +132,14 @@ class TestXMLRPCController(TestWSGIController):
     def test_contenttype(self):
         response = self.xmlreq('system.methodHelp', ('longdoc',))
         assert self.response.header('Content-Type') == 'text/xml'
-
+    
     def test_start_response(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, 'start_response')
-
+    
     def test_private_func(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, '_private')
-
+    
     def test_var(self):
         self.assertRaises(xmlrpclib.Fault, self.xmlreq, 'foo')
+    
+
