@@ -1,8 +1,8 @@
-from pylons import c, g, cache, request, session
+from pylons import tmpl_context as c, app_globals, cache, request, session
 from pylons.controllers import WSGIController, XMLRPCController
 from pylons.controllers.util import abort, redirect_to, etag_cache
 from pylons.decorators import jsonify, validate
-from pylons.templating import render, render_response
+from pylons.templating import render_mako as render
 from pylons.i18n import N_, _, ungettext
 import projectname.model as model
 import projectname.lib.helpers as h
