@@ -3,4 +3,4 @@
 Consists of functions to typically be used within templates, but also
 available to Controllers. This module is available to both as 'h'.
 """
-from webhelpers.rails import *
+from pylons.controllers.util import url_for
